<?php
/**
 * Class eShopMagic_HTMLElement 
 * @description A single HTML element.
 * @author __construct
 */

if ( !class_exists( 'eShopMagic_HTMLElement' ) ) {
class eShopMagic_HTMLElement {

	private $children = Array( );
	private $attrs = Array( );
	private $type = '';

	public function __construct( $type, $attrlist = '', $childlist = Array( ) ) {
		$this->type = $type;
		$this->attrs = $this->parseAttrString( $attrlist );

		$children = Array( );
		if ( is_array( $childlist ) ) {
			$children = $childlist;
		} elseif ( strlen( $childlist ) ) {
			$children = Array( $childlist );	
		}
		$this->children = $children;
	}

	public function __get( $name ) {
		return ( isset( $this->attrs[$name] ) ) ? $this->attrs[$name] : '';
	}
	
	public function __set( $name, $value ) {
		$this->attrs[$name] = $value;
	}

	public function getElementType( ) {
		return $this->type;
	}

	public function setAttrs( $attrs ) {
		if ( !is_array( $attrs ) ) {
			$attrs = $this->parseAttrString( $attrs );
		}

		foreach( $attrs as $key => $val ) {
			$this->attrs[$key] = $val;
		}
	}

	public function appendChild( $child ) {
		$this->children[] = $child;
	}

	public function appendChildren( ) {
		$children = func_get_args( );
		foreach( $children as $child ) {
			$this->appendChild( $child );
		}
	}

	public function getChildren( ) {
		return $this->children;
	}

	public function setChildren( Array $children ) {
		$this->children = $children;
	}

	public function __tostring( ) {
		$inner = '';
		if ( is_array( $this->children ) ) {
			foreach( $this->children as $child ) {
				$inner .= $child;
			}
		}
		
		$attrs = $this->getAttributeString( $this->attrs );

		return "<{$this->type}{$attrs}>{$inner}</{$this->type}>";
	}
	
	private function getAttributeString( $attrs = Array( ) ) {
		$attributeString = '';
		foreach( $attrs as $key => $value ) {
			$value = htmlspecialchars( $value, ENT_QUOTES ); 

			if ( strlen( $value ) ) {
				$attributeString .= " {$key}='{$value}'";
			} else {
				if ( $key != 'checked' ) $attributeString .= " {$key}";
			}
		}
		return $attributeString;
	}
	
	private function parseAttrString( $attrstring ) {
		$attrs = Array( );
		$items = ( !empty( $attrstring ) ) ? explode( ',', $attrstring ) : Array( );
		foreach( $items as $i ) {
			list( $key, $val ) = preg_split( '/:/', trim( $i ), 2 );
			$keys = explode( '+', $key );
			foreach( $keys as $k ) {
				$attrs[$k] = $val;
			}
		}
		return $attrs;
	}

} // End class Element
} // End if

